<!-- file: INSTALL.md -->


INSTALL
=======

This document describes building and installing S4lt *from sources*.

Actually, S4lt consists of the single source file, `src/s4lt.m4`, which does not require any
building. You can manually extract this file from the source tarball and copy to destination
directory. However, build infrastructure can be used for testing, installing, building source
tarball and rpm package.


Prequisities
------------

Following tools are used for building and testing:

*   `make`
*   `autom4te`  (testing S4lt)
*   `perl`      (testing S4lt)
*   `rpmbuild`  (building RPM packages)
*   `rpmlint`   (testing RPM packages)


Unpacking
---------

Unpack the source tarball first:

    $ tar xaf s4lt-VERSION.tar.gz
    $ cd s4lt-VERSION

In-source build is discouraged, so create build directory and change to it before running
`configure`:

    $ mkdir _build
    $ cd _build

All the intermediate files and results (including RPM packages) will be located in the build
directory.


Configuring
-----------

Run `configure` from within the build directory:

    $ ../configure

These `configure` options may be useful:

`--prefix=dir`

:   Specify installation prefix.

`--disable-test`

:   Do not look for tools used for testing (e. g. `autoconf`, `perl`, `rpmlint`), do not run tests.

`--disable-rpm`

:   Do not look for `rpmbuild` tool, do not build RPM packages.

Run `configure --help` to see list of all available options.

**Note:** `configure` uses prefix `/usr/local` by default. Autoconf will be able to find and use
S4lt if they both installed using *the same* prefix. If you are going to add S4lt to your
system-wide Autoconf (which is likely installed to `/usr`), you have to configure S4lt properly.
However, building and installing an RPM package is preferred method to achieve it (for RPM-based
distros, of course).


Building
--------

Then run `make`:

    $ make

By default, RPM packages are not built. To build them, specify target `rpm` in the command line:

    $ make rpm

All GNU-Automake-standard targets (`check`, `dist`, etc) are supported. There are some non-standard
convenience targets, run

    $ make help

to see them.


Testing
-------

To execute tests, run

    $ make check


Installing
----------

To install freshly built RPM package (this is preferred method for RPM-based distros) use either
`dnf`:

    $ sudo dnf install ./s4lt-VERSION-RELEASE.noarch.rpm

or `rpm` command:

    $ sudo rpm -i ./s4lt-VERSION-RELEASE.noarch.rpm

Installing S4lt through `make` also possible:

    $ sudo make install

But remember about prefix: system-wide Autoconf (usually installed into `/usr`) will not able to
find and use S4lt installed by default into `/usr/local`.


<!-- end of file -->
