#!/bin/sh
#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/check-rpm-rpmlint.sh
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

eval $PROLOGUE

test $# -lt 1       && die "No argument"
test $# -gt 1       && die "Too many arguments"
test -z "$RPMLINT"  && skip "RPMLINT is not set"

mktmpdir
log="$tmpdir/$( basename "$1" ).log"

#   rpmlint exit statuses:
#        0  No errors.
#        1  Unspecified error.
#        2  Interrupted.
#       64  One or more error message printed.
#       66  Badness threshold exceeded.

#   grep returns status 0 if there are some output, 1 if output is empty and > 1 in case of errors.
#   In our case empty output means everything is ok (no warning no error messages), so we need to
#   invert grep status, but not ignore errors.

check=0
grep=0

$RPMLINT --file="$SRCDIR/rpm/rpmlint.conf" "$@" > "$log" 2>&1 || check=$?

# `rpmlint.conf` can filters out erors and warnings but passes such message:
w='1 packages and 0 specfiles checked;'

grep -E -v -e "($w)" "$log" || grep=$?

if test $check -eq 64 || test $check -eq 66; then
    fail        # Test failure.
fi
if test $check -ne 0; then
    die         # Error, not test failure.
fi
if test $grep -eq 0; then
    # grep passed some lines through. They are warnings, let us report them as test failure.
    fail        # Test failure.
fi
pass

# end of file #
