#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/prologue.sh
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

script=$( basename "$0" )
tmpdir="$script.tmp"

say() {
    local line
    for line in "$@"; do
        echo "$script: $line" >&2
    done
}

pass() {
    say "$@"
    exit 0
}

fail() {
    say "$@"
    exit 1
}

die() {
    say "$@"
    exit 99
}

skip() {
    say "$@"
    exit 77
}

mktmpdir() {
    test -d "$tmpdir" || mkdir "$tmpdir"
}

# grep returns 0 (true) if there are some lines in output.
# grep returns 1 (false) if output is empty.
# grep returns > 1 if there some errors.
# Tests will likely check for 0 and 1, but status > 1 is definetely error and should be reported
# immediately. It simplifies tests.
grep() {
    test -n $GREP || skip "GREP is not set."
    local rc=0
    # It looks like a bash 4.3.42 bug: if set -e, non-exitent command returns status 1, not 127.
    # Let us temporary reset errexit option.
    set +e
    $GREP "$@" || rc=$?
    set -e
    test $rc -gt 1 && die "grep error"
    return $rc
}

# end of file #
