#!/bin/bash
#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/tar.sh
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

#   Find modification time of the last edited file listed in the MANIFEST.txt, and use this time
#   for all files in the tarball.
#
#   Usage: bin/tar.sh TAR OPTION... ARGUMENT...
#
#   Note:

set -eo pipefail

function die() {
    for line in "$@"; do
        echo "${0##*/}: $line" >&2
    done
    exit 99
};

[[ -n "$SRCDIR" ]] || die "SRCDIR is not set."

manifest="$SRCDIR/MANIFEST.txt"

[[ -e "$manifest" ]] || die "$manifest does not exist."
[[ -f "$manifest" ]] || die "$manifest is not a file."

# Find modification time of the most recently edited file listed in the MANIFEST.txt:
mtime=0
while read line; do
    if [[ -z $line ]]; then
        : # Ignore empty lines.
    elif [[ $line =~ ^\# ]]; then
        : # Ignore comments.
    else
        m=$( stat -c %Y $SRCDIR/$line ) # Modification time, seconds since epoch.
        if [[ $m -gt $mtime ]]; then
            mtime=$m
        fi
    fi
done < "$manifest"

# mtime is in seconds since the epoch; convert it to ISO date:
mtime=$( date --date="@$mtime" +%FT%T )

# Ok, run tar with extra options:
"$@" --mtime="$mtime"

# end of file #
