<!-- file: m4/md.m4 -->
m4_divert(-1)
#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: m4/md.m4
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

#
#   M4 macros for embedding microdata into markdown source (to generate html with microdata).
#   Abot microdata see:
#
#       https://en.wikipedia.org/wiki/Microdata_(HTML)
#       http://www.w3.org/TR/microdata/
#       http://schema.org/
#
#   Usage example:
#
#       It is made for MD_KEYWORD({{ GNU/Linux }}) based on MD_KEYWORD({{ glibc }}).
#       MD_OFFER({{ Tapper is MD_PRICE({{ free }}) software. }})
#


m4_changequote({{, }})


#   _md_ignore()
#   ------------------------------------------------------------------------------------------------
#   Ignore all the arguments, expand to empty string.

m4_define({{_md_ignore}}, {{}})


#   _md_ltrim(STRING)
#   ------------------------------------------------------------------------------------------------
#   Trims leading whitespaces from STRING. Non-leading whitespace remains intact. Returns quoted
#   string.

m4_define({{_md_ltrim}}, {{m4_patsubst({{{{$1}}}}, {{\`\(..\)\s*}}, {{\1}})}})


#   _md_rtrim(STRING)
#   ------------------------------------------------------------------------------------------------
#   Trims trailing whitespaces from STRING. Non-trailing whitespace remains intact. Returns quoted
#   string.

m4_define({{_md_rtrim}}, {{m4_patsubst({{{{$1}}}}, {{\s*\(..\)\'}}, {{\1}})}})


#   _md_trim(STRING)
#   ------------------------------------------------------------------------------------------------
#   Trims both leading and trailing whitespaces from STRING. Inner whitespace remains intact.
#   Returns quoted string.

m4_define({{_md_trim}}, {{_md_ltrim(_md_rtrim({{$1}}))}})


#   _md_norm(STRING)
#   ------------------------------------------------------------------------------------------------
#   Trims both leading and trailing whitespaces. Every double tilda followed by newline is deleted
#   with the following whitespaces. Returns quoted string.

m4_define({{_md_norm}}, {{_md_trim(m4_patsubst({{{{$1}}}}, {{~~
\s*}}, {{{{}}}}))}})


#   _md_demark(STRING)
#   ------------------------------------------------------------------------------------------------
#   Strips markdown formatting from string. Frankly, it just deletes all square brackets, asterisks,
#   and grave accents. Returns quoted string.

m4_define({{_md_demark}}, {{m4_translit({{{{$1}}}}, {{[]*`}})}})m4_dnl   # `


#   ================================================================================================
#   m4 macro wrappers.
#   ================================================================================================

#   Each of these _md_* macros normalizes every argument and invokes appropriate m4_ macro.

m4_define({{_md_define}}, {{m4_define(_md_norm({{$1}}), _md_norm({{$2}}))}})

_md_define({{ _md_ifdef    }},{{ m4_ifdef(    _md_norm({{$1}}), _md_norm({{$2}}), _md_norm({{$3}})) }})
_md_define({{ _md_ifelse   }},{{ m4_ifelse(   _md_norm({{$1}}), _md_norm({{$2}}), _md_norm({{$3}}), _md_norm({{$4}})) }})
_md_define({{ _md_pushdef  }},{{ m4_pushdef(  _md_norm({{$1}}), _md_norm({{$2}})) }})
_md_define({{ _md_popdef   }},{{ m4_popdef(   _md_norm({{$1}})) }})
_md_define({{ _md_divert   }},{{ m4_divert(   _md_norm({{$1}})) }})
_md_define({{ _md_undivert }},{{ m4_undivert( _md_norm({{$1}})) }})


#   ================================================================================================
#   m4sugar-like macros.
#   ================================================================================================

_md_define({{ _md_expand }},{{ $0_(--==<<($1)>>==--) }})
_md_define({{ _md_expand_ }},{{
    m4_changequote({{--==<<(}}, {{)>>==--}})$1m4_changequote({{, }})
}})


#   MD_ENABLE
#   ------------------------------------------------------------------------------------------------
#   If 1, MD_* macros generate microdata tags. Otherwise, plain text is generated. By default
#   microdata generation is enabled. Microdata can be turned off either from command line:
#       $ m4 -D MD_ENABLE=0 ...
#   or from code:
#       m4_define({{MD_ENABLE}},{{0}})

_md_ifdef({{ MD_ENABLE }},{{ }},{{ _md_define({{ MD_ENABLE }},{{ 1 }}) }})


#   _md_divnum_kword
#   ------------------------------------------------------------------------------------------------
#   Number of diversion to accumulate keywords in. If < 0, keywords accumulation is disabled.

_md_ifdef({{ _md_divnum_kword }},{{ }},{{ _md_define({{ _md_divnum_kword }},{{ 1001 }}) }})


#   _md_divnum_descr
#   ------------------------------------------------------------------------------------------------
#   Number of diversion to write plain text description to.

_md_ifdef({{ _md_divnum_descr }},{{ }},{{ _md_define({{ _md_divnum_descr }},{{ 1002 }}) }})


#   _md_prop( PROPERTY, ITEM_TYPE, [MACHINE_READABLE,] USER_VISIBLE )
#   ---------------------------------------------------------------------------------------------
#   Workhorse of all _MD_* macros. Do not use it directly in markdown code. IF MD_ENABLE is 1, the
#   macro generates html `span` element (<span>...</span>). otherwise macro expansion is empty.
#   PROPERTY is value of `itemprop` attribute: <span itemprop="PROPERTY">. If ITEM_TYPE is not
#   empty, `span` tag will have `itemscope` and `itemtype` attributes: <span ... itemscope
#   itemtype="http://schema.org/ITEM_TYPE>. The next two arguments defines value of `content`
#   attribute and content of the `span` element: <span ...
#   content="MACHINE_READABLE">USER_VISIBLE</span>. If the 4th argument is empty, MACHINE_READABLE
#   is considered as omitted and `content` attribute is not generated. If you want to have `span`
#   with empty content, use dash (minus, `-`) as the 4th argument.
#
#   TODO: Swap MACHINE_READABLE and USER_VISIBLE? I originally choosed this order because it
#       is the same as their order in generated HTML code: <span itemprop itemtype="ITEM_TYPE"
#       content="MACHINE_READABLE">USER_VISIBLE</span>. However, it seems code of macro will be
#       simpler if MACHINE_READABLE and USER_VISIBLE are swapped.
#
#   TODO: Write tests?

_md_define({{ _md_prop }},{{
    _md_ifelse(
        MD_ENABLE
    ,{{
        1
    }},{{
        <span itemprop="_md_trim({{ $1 }})"~~
        _md_ifelse({{
            $2
        }},{{
        }},{{
        }},{{
            {{ }}itemscope itemtype="http://schema.org/_md_trim({{ $2 }})"
        }})~~
        _md_ifelse({{
            $4
        }},{{
        }},{{
        }},{{
            _md_ifelse({{
                $3
            }},{{
            }},{{
            }},{{
                {{ }}content="_md_trim( _md_demark({{ $3 }}) )"
            }})
        }})~~
        >
    }})~~
    _md_ifelse({{
        $4
    }},{{
    }},{{
        $3
    }},{{
        _md_ifelse({{
            $4
        }},{{
            -
        }},{{
        }},{{
            $4
        }})~~
    }})~~
    _md_ifelse(
        MD_ENABLE
    ,{{
        1
    }},{{
        </span>
    }})
}})

#   MD_PROPERTY( [MACHINE_READABLE,] USER_VISIBLE )
#   ------------------------------------------------------------------------------------------------

_md_define({{ MD_NAME             }},{{ _md_prop({{ name                   }},{{        }},{{ $1 }},{{ $2 }}) }})m4_dnl
_md_define({{ MD_CATEGORY         }},{{ _md_prop({{ applicationCategory    }},{{        }},{{ $1 }},{{ $2 }}) }})m4_dnl
_md_define({{ MD_SUB_CATEGORY     }},{{ _md_prop({{ applicationSubCategory }},{{        }},{{ $1 }},{{ $2 }}) }})m4_dnl
_md_define({{ MD_PRICE            }},{{ _md_prop({{ price                  }},{{        }},{{ $1 }},{{ $2 }}) }})m4_dnl
_md_define({{ MD_PRICE_CURRENCY   }},{{ _md_prop({{ priceCurrency          }},{{        }},{{ $1 }},{{ $2 }}) }})m4_dnl
_md_define({{ MD_COPYRIGHT_YEAR   }},{{ _md_prop({{ copyrightYear          }},{{        }},{{ $1 }},{{ $2 }}) }})m4_dnl
_md_define({{ MD_HEADLINE         }},{{ _md_prop({{ headline               }},{{        }},{{ $1 }},{{ $2 }}) }})m4_dnl
_md_define({{ MD_LICENSE          }},{{ _md_prop({{ license                }},{{        }},{{    }},{{ _md_trim({{$1}}) }}) }})m4_dnl
_md_define({{ MD_COPYRIGHT_HOLDER }},{{ _md_prop({{ copyrightHolder        }},{{ Person }},{{ $1 }},{{ $2 }}) }})m4_dnl
_md_define({{ MD_AUTHOR           }},{{ _md_prop({{ author                 }},{{ Person }},{{ $1 }},{{    }}) }})m4_dnl
_md_define({{ MD_OFFER            }},{{ _md_prop({{ offers                 }},{{ Offer  }},{{ $1 }},{{    }}) }})m4_dnl


#   MD_OPERATING_SYSTEM( [MACHINE_READABLE,] USER_VISIBLE )
#   ------------------------------------------------------------------------------------------------
#   MD_OPERATING_SYSTEM is a bit special: it generates `operationSystem` and `keywords` microdata.

_md_define({{ MD_OPERATING_SYSTEM }},{{
    _md_prop({{ operatingSystem }},{{ }},{{ $1 }},{{ MD_KEYWORD({{ $1 }},{{ $2 }}) }})
}})


#   MD_KEYWORD( [CONTENT_ATTR,] CONTENT_ELEM )
#   MD_KEYWORDS
#   ------------------------------------------------------------------------------------------------
#   MD_KEYWORD is special: it generates `keywords` microdata and appends CONTENT_ELEM to
#   _md_divnum_kword diversion (if diversion number >= 0).
#   MD_KEYWORDS undiverts plain comma-separated list of keywords.

_md_define({{ MD_KEYWORD }},{{
    _md_prop({{ keywords }},{{ }},{{ $1 }},{{ $2 }})~~
    _md_ifelse(
        m4_eval( _md_divnum_kword >= 0 )
    ,{{
        1
    }},{{
        _md_pushdef({{ _md_divnum }}, m4_divnum )~~
        _md_pushdef({{ MD_ENABLE }})~~
        m4_divert(_md_divnum_kword)~~
        _md_trim( _md_demark( _md_expand({{ $1 }}) ) )~~
        , ~~
        _md_divert( _md_divnum )~~
        _md_popdef({{ MD_ENABLE }})~~
        _md_popdef({{ _md_divnum }})
    }})
}})
_md_define({{ MD_KEYWORDS }},{{ _md_undivert( _md_divnum_kword ) }})


#   MD_DESCRIPTION( CONTENT )
#   MD_DESCRIPTION
#   ------------------------------------------------------------------------------------------------
#   MD_DESCRIPTION is special: it generates `description` microdata and saves plain text version of
#   CONTENT to _md_divnum_descr diversion (if diversion number >= 0). MD_DESCRIPTION with no
#   arguments undiverts plain text description.

_md_define({{ MD_DESCRIPTION }},{{
    _md_ifelse({{
        $#
    }},{{
        0
    }},{{
        _md_undivert( _md_divnum_descr )
    }},{{
        _md_prop({{ description }},{{ }},{{ $1 }},{{ }})~~
        _md_ifelse(
            m4_eval( _md_divnum_descr >= 0 )
        ,{{
            1
        }},{{
            _md_pushdef({{ MD_ENABLE }})~~
            _md_pushdef({{ _md_divnum_kword }},{{ -1 }})~~
            _md_pushdef({{ _md_divnum }}, m4_divnum )~~
            _md_divert( _md_divnum_descr )~~
            _md_trim( _md_demark( _md_expand({{ $1 }}) ) )~~
            _md_divert( _md_divnum )~~
            _md_popdef({{ _md_divnum }} )~~
            _md_popdef({{ _md_divnum_kword }})~~
            _md_popdef({{ MD_ENABLE }})
        }})
    }})
}})


#   MD_COMMENT( CONTENT )
#   ------------------------------------------------------------------------------------------------
#   MD_COMMENT creates a valid HTML comment. Tidy complains if a comment includes ajacent dashes.
#   This macro replaces series of adjacent dashes (U+002D) with series of Unicode minus signes
#   (U+2212). Unicode minus sign looks like dash (ASCII minus), but it is different character which
#   does not cause tidy errors.

_md_define({{ MD_COMMENT }},{{
    {{<!--}} m4_patsubst({{$1}},{{--+}},{{m4_patsubst({{\&}},{{-}},{{−}})}}) {{-->}}
}})


m4_changecom({{<!--}}, {{-->}})
m4_divert(0)m4_dnl

<!-- end of file -->
